@@ -75,10 +75,10 @@
  *   <example id="ClutterAlpha-script-example">
  *     <title>Defining a ClutterAlpha in ClutterScript</title>
  *     <para>The following JSON fragment defines a #ClutterAlpha
- *     using a #ClutterTimeline with id "timeline-01" and an alpha
+ *     using a #ClutterTimeline with id "sine-timeline" and an alpha
  *     function called my_sine_alpha(). The defined #ClutterAlpha
  *     instance can be reused in multiple #ClutterBehaviour
- *     definitions.</para>
+ *     definitions or for #ClutterAnimation definitions.</para>
  *     <programlisting>
  *  {
  *    "id" : "sine-alpha",
@@ -91,6 +91,10 @@
  *  }
  *     </programlisting>
  *   </example>
+ *   <para>For the way to define the #ClutterAlpha:mode property
+ *   inside a ClutterScript fragment, see <link
+ *   linkend="clutter-AnimationMode-Script">the corresponding section</link>
+ *   in #ClutterAnimation.</para>
  * </refsect2>
  *
  * Since: 0.2
@@ -244,7 +248,7 @@ resolve_alpha_func (const gchar *name)
   CLUTTER_NOTE (SCRIPT, "Looking up '%s' alpha function", name);
 
   if (G_UNLIKELY (module == NULL))
-    module = g_module_open (NULL, G_MODULE_BIND_LAZY);
+    module = g_module_open (NULL, 0);
 
   if (g_module_symbol (module, name, (gpointer) &func))
     {
