@@ -18,6 +18,7 @@ clutter_actor_flags_get_type (void)
         { CLUTTER_ACTOR_REALIZED, "CLUTTER_ACTOR_REALIZED", "realized" },
         { CLUTTER_ACTOR_REACTIVE, "CLUTTER_ACTOR_REACTIVE", "reactive" },
         { CLUTTER_ACTOR_VISIBLE, "CLUTTER_ACTOR_VISIBLE", "visible" },
+        { CLUTTER_ACTOR_NO_LAYOUT, "CLUTTER_ACTOR_NO_LAYOUT", "no-layout" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
@@ -53,6 +54,62 @@ clutter_allocation_flags_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-bin-layout.h" */
+#include "./clutter-bin-layout.h"
+
+GType
+clutter_bin_alignment_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_BIN_ALIGNMENT_FIXED, "CLUTTER_BIN_ALIGNMENT_FIXED", "fixed" },
+        { CLUTTER_BIN_ALIGNMENT_FILL, "CLUTTER_BIN_ALIGNMENT_FILL", "fill" },
+        { CLUTTER_BIN_ALIGNMENT_START, "CLUTTER_BIN_ALIGNMENT_START", "start" },
+        { CLUTTER_BIN_ALIGNMENT_END, "CLUTTER_BIN_ALIGNMENT_END", "end" },
+        { CLUTTER_BIN_ALIGNMENT_CENTER, "CLUTTER_BIN_ALIGNMENT_CENTER", "center" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterBinAlignment"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
+/* enumerations from "./clutter-box-layout.h" */
+#include "./clutter-box-layout.h"
+
+GType
+clutter_box_alignment_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_BOX_ALIGNMENT_START, "CLUTTER_BOX_ALIGNMENT_START", "start" },
+        { CLUTTER_BOX_ALIGNMENT_END, "CLUTTER_BOX_ALIGNMENT_END", "end" },
+        { CLUTTER_BOX_ALIGNMENT_CENTER, "CLUTTER_BOX_ALIGNMENT_CENTER", "center" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterBoxAlignment"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-event.h" */
 #include "./clutter-event.h"
 
@@ -255,6 +312,32 @@ clutter_feature_flags_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-flow-layout.h" */
+#include "./clutter-flow-layout.h"
+
+GType
+clutter_flow_orientation_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_FLOW_HORIZONTAL, "CLUTTER_FLOW_HORIZONTAL", "horizontal" },
+        { CLUTTER_FLOW_VERTICAL, "CLUTTER_FLOW_VERTICAL", "vertical" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterFlowOrientation"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-main.h" */
 #include "./clutter-main.h"
 
@@ -671,6 +754,29 @@ clutter_font_flags_get_type (void)
 
   return g_enum_type_id__volatile;
 }
+GType
+clutter_text_direction_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_TEXT_DIRECTION_DEFAULT, "CLUTTER_TEXT_DIRECTION_DEFAULT", "default" },
+        { CLUTTER_TEXT_DIRECTION_LTR, "CLUTTER_TEXT_DIRECTION_LTR", "ltr" },
+        { CLUTTER_TEXT_DIRECTION_RTL, "CLUTTER_TEXT_DIRECTION_RTL", "rtl" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterTextDirection"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
 
 /* enumerations from "./clutter-units.h" */
 #include "./clutter-units.h"
@@ -687,6 +793,7 @@ clutter_unit_type_get_type (void)
         { CLUTTER_UNIT_EM, "CLUTTER_UNIT_EM", "em" },
         { CLUTTER_UNIT_MM, "CLUTTER_UNIT_MM", "mm" },
         { CLUTTER_UNIT_POINT, "CLUTTER_UNIT_POINT", "point" },
+        { CLUTTER_UNIT_CM, "CLUTTER_UNIT_CM", "cm" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
