@@ -67,6 +67,7 @@
 #include "clutter-stage-window.h"
 #include "clutter-version.h" 	/* For flavour */
 #include "clutter-id-pool.h"
+#include "clutter-container.h"
 
 #include "cogl/cogl.h"
 
@@ -95,6 +96,7 @@ struct _ClutterStagePrivate
   guint is_user_resizable      : 1;
   guint use_fog                : 1;
   guint throttle_motion_events : 1;
+  guint use_alpha              : 1;
 };
 
 enum
@@ -109,7 +111,8 @@ enum
   PROP_TITLE,
   PROP_USER_RESIZE,
   PROP_USE_FOG,
-  PROP_FOG
+  PROP_FOG,
+  PROP_USE_ALPHA
 };
 
 enum
@@ -192,6 +195,10 @@ clutter_stage_allocate (ClutterActor           *self,
     {
       ClutterActorClass *klass;
 
+      /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
+       * need to manually keep it informed of the current window size */
+      _cogl_onscreen_clutter_backend_set_size (width, height);
+
       CLUTTER_NOTE (LAYOUT,
                     "Following allocation to %dx%d (origin %s)",
                     width, height,
@@ -210,6 +217,10 @@ clutter_stage_allocate (ClutterActor           *self,
 
       _clutter_stage_window_get_geometry (priv->impl, &geom);
 
+      /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
+       * need to manually keep it informed of the current window size */
+      _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
+
       override.x1 = 0;
       override.y1 = 0;
       override.x2 = geom.width;
@@ -234,14 +245,25 @@ clutter_stage_paint (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
   CoglColor stage_color;
+  guint8 real_alpha;
 
   CLUTTER_NOTE (PAINT, "Initializing stage paint");
 
+  /* composite the opacity to the stage color */
+  real_alpha = clutter_actor_get_opacity (self)
+             * priv->color.alpha
+             / 255;
+
+  /* we use the real alpha to clear the stage if :use-alpha is
+   * set; the effect depends entirely on how the Clutter backend
+   */
   cogl_color_set_from_4ub (&stage_color,
                            priv->color.red,
                            priv->color.green,
                            priv->color.blue,
-                           priv->color.alpha);
+                           priv->use_alpha ? real_alpha
+                                           : 255);
+  cogl_color_premultiply (&stage_color);
   cogl_clear (&stage_color,
 	      COGL_BUFFER_BIT_COLOR |
 	      COGL_BUFFER_BIT_DEPTH);
@@ -261,11 +283,6 @@ clutter_stage_paint (ClutterActor *self)
   else
     cogl_disable_fog ();
 
-#if 0
-  CLUTTER_NOTE (PAINT, "Proxying the paint to the stage implementation");
-  _clutter_stage_window_paint (priv->impl);
-#endif
-
   /* this will take care of painting every child */
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->paint (self);
 }
@@ -274,12 +291,13 @@ static void
 clutter_stage_pick (ClutterActor       *self,
 		    const ClutterColor *color)
 {
-  /* Paint nothing, cogl_paint_init() effectively paints the stage
-   * silhouette for us - see _clutter_do_pick().
-   * Chain up to the groups paint howerer so our children get picked
-   * - clutter_group_pick
+  /* Note: we don't chain up to our parent as we don't want any geometry
+   * emitted for the stage itself. The stage's pick id is effectively handled
+   * by the call to cogl_clear done in clutter-main.c:_clutter_do_pick_async()
    */
-  CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->pick (self, color);
+  clutter_container_foreach (CLUTTER_CONTAINER (self),
+                             CLUTTER_CALLBACK (clutter_actor_paint),
+                             NULL);
 }
 
 static void
@@ -302,8 +320,26 @@ clutter_stage_realize (ClutterActor *self)
    */
   if (is_realized)
     {
-      CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
-      clutter_stage_ensure_current (CLUTTER_STAGE (self));
+      GError *error = NULL;
+      ClutterBackend *backend = clutter_get_default_backend ();
+
+      /* We want to select the context without calling
+         clutter_backend_ensure_context so that it doesn't call any
+         Cogl functions. Otherwise it would create the Cogl context
+         before we get a chance to check whether the GL version is
+         valid */
+      _clutter_backend_ensure_context_internal (backend, CLUTTER_STAGE (self));
+
+      /* Make sure Cogl can support the driver */
+      if (!_cogl_check_driver_valid (&error))
+        {
+          g_warning ("The GL driver is not supported: %s",
+                     error->message);
+          g_clear_error (&error);
+          CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
+        }
+      else
+        CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
     }
   else
     CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
@@ -632,6 +668,10 @@ clutter_stage_set_property (GObject      *object,
       clutter_stage_set_fog (stage, g_value_get_boxed (value));
       break;
 
+    case PROP_USE_ALPHA:
+      clutter_stage_set_use_alpha (stage, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -684,6 +724,10 @@ clutter_stage_get_property (GObject    *gobject,
       g_value_set_boxed (value, &priv->fog);
       break;
 
+    case PROP_USE_ALPHA:
+      g_value_set_boolean (value, priv->use_alpha);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -696,11 +740,29 @@ clutter_stage_dispose (GObject *object)
   ClutterStage        *stage = CLUTTER_STAGE (object);
   ClutterStagePrivate *priv = stage->priv;
   ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
+  ClutterMainContext  *context;
+  GList               *l, *next;
 
   clutter_actor_hide (CLUTTER_ACTOR (object));
 
   _clutter_stage_manager_remove_stage (stage_manager, stage);
 
+  context = _clutter_context_get_default ();
+
+  /* Remove any pending events for this stage from the event queue */
+  for (l = context->events_queue->head; l; l = next)
+    {
+      ClutterEvent *event = l->data;
+
+      next = l->next;
+
+      if (event->any.stage == stage)
+        {
+          g_queue_delete_link (context->events_queue, l);
+          clutter_event_free (event);
+        }
+    }
+
   if (priv->impl != NULL)
     {
       CLUTTER_NOTE (BACKEND, "Disposing of the stage implementation");
@@ -893,6 +955,24 @@ clutter_stage_class_init (ClutterStageClass *klass)
                               CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_FOG, pspec);
 
+  /**
+   * ClutterStage:use-alpha:
+   *
+   * Whether the #ClutterStage should honour the alpha component of the
+   * #ClutterStage:color property when painting. If Clutter is run under
+   * a compositing manager this will result in the stage being blended
+   * with the underlying window(s)
+   *
+   * Since: 1.2
+   */
+  pspec = g_param_spec_boolean ("use-alpha",
+                                "Use Alpha",
+                                "Whether to honour the alpha component of "
+                                "the stage color",
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_USE_ALPHA, pspec);
+
   /**
    * ClutterStage::fullscreen
    * @stage: the stage which was fullscreened
@@ -2083,3 +2163,53 @@ clutter_stage_get_throttle_motion_events (ClutterStage *stage)
 
   return stage->priv->throttle_motion_events;
 }
+
+/**
+ * clutter_stage_set_use_alpha:
+ * @stage: a #ClutterStage
+ * @use_alpha: whether the stage should honour the opacity or the
+ *   alpha channel of the stage color
+ *
+ * Sets whether the @stage should honour the #ClutterActor:opacity and
+ * the alpha channel of the #ClutterStage:color
+ *
+ * Since: 1.2
+ */
+void
+clutter_stage_set_use_alpha (ClutterStage *stage,
+                             gboolean      use_alpha)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  if (priv->use_alpha != use_alpha)
+    {
+      priv->use_alpha = use_alpha;
+
+      clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+
+      g_object_notify (G_OBJECT (stage), "use-alpha");
+    }
+}
+
+/**
+ * clutter_stage_get_use_alpha:
+ * @stage: a #ClutterStage
+ *
+ * Retrieves the value set using clutter_stage_set_use_alpha()
+ *
+ * Return value: %TRUE if the stage should honour the opacity and the
+ *   alpha channel of the stage color
+ *
+ * Since: 1.2
+ */
+gboolean
+clutter_stage_get_use_alpha (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  return stage->priv->use_alpha;
+}
