@@ -38,38 +38,33 @@ typedef struct
   GLubyte c[4];
 } CoglTextureGLVertex;
 
-typedef struct
-{
-  CoglBufferTarget target;
-  CoglHandle offscreen;
-} CoglDrawBufferState;
-
 typedef struct
 {
   /* Features cache */
   CoglFeatureFlags  feature_flags;
   gboolean          features_cached;
 
+  CoglHandle        default_material;
+
   /* Enable cache */
   gulong            enable_flags;
   guint8            color_alpha;
 
   gboolean          enable_backface_culling;
+  CoglFrontWinding  flushed_front_winding;
 
   gboolean          indirect;
 
+  /* A few handy matrix constants */
+  CoglMatrix        identity_matrix;
+  CoglMatrix        y_flip_matrix;
+
   /* Client-side matrix stack or NULL if none */
   CoglMatrixMode    flushed_matrix_mode;
-  CoglMatrixStack  *projection_stack;
-  CoglMatrixStack  *modelview_stack;
-
   GList            *texture_units;
 
-  /* Cache of inverse projection matrix */
-  float            inverse_projection[16];
-
   /* Materials */
-  CoglHandle        default_material;
+  CoglHandle        simple_material;
   CoglHandle	    source_material;
 
   /* Textures */
@@ -91,11 +86,11 @@ typedef struct
   GArray           *current_layers;
   guint             n_texcoord_arrays_enabled;
 
-  /* Framebuffer objects */
-  GSList           *draw_buffer_stack;
-
-  /* Clip stack */
-  CoglClipStackState clip;
+  /* Framebuffers */
+  GSList           *framebuffer_stack;
+  CoglHandle        window_buffer;
+  gboolean          dirty_bound_framebuffer;
+  gboolean          dirty_gl_viewport;
 
   /* Primitives */
   floatVec2         path_start;
@@ -114,9 +109,6 @@ typedef struct
 
   gboolean          in_begin_gl_block;
 
-  guint             viewport_width;
-  guint             viewport_height;
-
   CoglHandle        texture_download_material;
 
   CoglContextDriver drv;
