@@ -138,6 +138,7 @@
 #include "cogl-texture-private.h"
 #include "cogl-material-private.h"
 #include "cogl-primitives.h"
+#include "cogl-framebuffer-private.h"
 
 #define PAD_FOR_ALIGNMENT(VAR, TYPE_SIZE) \
   (VAR = TYPE_SIZE + ((VAR - 1) & ~(TYPE_SIZE - 1)))
@@ -149,13 +150,14 @@
 
 #if defined (HAVE_COGL_GL)
 
-#define glGenBuffers ctx->drv.pf_glGenBuffersARB
-#define glBindBuffer ctx->drv.pf_glBindBufferARB
-#define glBufferData ctx->drv.pf_glBufferDataARB
-#define glBufferSubData ctx->drv.pf_glBufferSubDataARB
-#define glDeleteBuffers ctx->drv.pf_glDeleteBuffersARB
-#define glMapBuffer ctx->drv.pf_glMapBufferARB
-#define glUnmapBuffer ctx->drv.pf_glUnmapBufferARB
+#define glGenBuffers ctx->drv.pf_glGenBuffers
+#define glBindBuffer ctx->drv.pf_glBindBuffer
+#define glBufferData ctx->drv.pf_glBufferData
+#define glBufferSubData ctx->drv.pf_glBufferSubData
+#define glGetBufferSubData ctx->drv.pf_glGetBufferSubData
+#define glDeleteBuffers ctx->drv.pf_glDeleteBuffers
+#define glMapBuffer ctx->drv.pf_glMapBuffer
+#define glUnmapBuffer ctx->drv.pf_glUnmapBuffer
 #define glActiveTexture ctx->drv.pf_glActiveTexture
 #define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
 #ifndef GL_ARRAY_BUFFER
@@ -174,9 +176,9 @@
 
 #if defined (HAVE_COGL_GL)
 
-#define glVertexAttribPointer ctx->drv.pf_glVertexAttribPointerARB
-#define glEnableVertexAttribArray ctx->drv.pf_glEnableVertexAttribArrayARB
-#define glDisableVertexAttribArray ctx->drv.pf_glEnableVertexAttribArrayARB
+#define glVertexAttribPointer ctx->drv.pf_glVertexAttribPointer
+#define glEnableVertexAttribArray ctx->drv.pf_glEnableVertexAttribArray
+#define glDisableVertexAttribArray ctx->drv.pf_glEnableVertexAttribArray
 #define MAY_HAVE_PROGRAMABLE_GL
 
 #elif defined (HAVE_COGL_GLES2)
@@ -1664,6 +1666,11 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
     }
   ctx->n_texcoord_arrays_enabled = max_texcoord_attrib_unit + 1;
 
+  /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
+   * as the material state) when flushing the clip stack, so should
+   * always be done first when preparing to draw. */
+  _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
+
   options.flags =
     COGL_MATERIAL_FLUSH_FALLBACK_MASK |
     COGL_MATERIAL_FLUSH_DISABLE_MASK;
@@ -1677,6 +1684,7 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
   cogl_enable (enable_flags);
+  _cogl_flush_face_winding ();
 }
 
 static void
@@ -1753,15 +1761,11 @@ cogl_vertex_buffer_draw (CoglHandle       handle,
     return;
 
   _cogl_journal_flush ();
-  cogl_clip_ensure ();
 
   buffer = _cogl_vertex_buffer_pointer_from_handle (handle);
 
-  cogl_clip_ensure ();
-  _cogl_flush_matrix_stacks ();
   enable_state_for_drawing_buffer (buffer);
 
-  /* FIXME: flush cogl cache */
   GE (glDrawArrays (mode, first, count));
 
   disable_state_for_drawing_buffer (buffer);
@@ -1885,7 +1889,6 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
     return;
 
   _cogl_journal_flush ();
-  cogl_clip_ensure ();
 
   buffer = _cogl_vertex_buffer_pointer_from_handle (handle);
 
@@ -1894,8 +1897,6 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
 
   indices = _cogl_vertex_buffer_indices_pointer_from_handle (indices_handle);
 
-  cogl_clip_ensure ();
-  _cogl_flush_matrix_stacks ();
   enable_state_for_drawing_buffer (buffer);
 
   byte_offset = indices_offset * get_indices_type_size (indices->type);
@@ -1905,7 +1906,6 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
     GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER,
                       GPOINTER_TO_UINT (indices->vbo_name)));
 
-  /* FIXME: flush cogl cache */
   GE (glDrawRangeElements (mode, min_index, max_index,
                            count, indices->type, (void *)byte_offset));
 
