@@ -38,6 +38,8 @@
 #include "cogl-context.h"
 #include "cogl-material-private.h"
 #include "cogl-winsys.h"
+#include "cogl-framebuffer-private.h"
+#include "cogl-matrix-private.h"
 
 #if defined (HAVE_COGL_GLES2) || defined (HAVE_COGL_GLES)
 #include "cogl-gles2-wrapper.h"
@@ -95,8 +97,8 @@ cogl_get_proc_address (const gchar* name)
 
   /* this should find the right function if the program is linked against a
    * library providing it */
-  if (module == NULL)
-    module = g_module_open (NULL, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
+  if (G_UNLIKELY (module == NULL))
+    module = g_module_open (NULL, 0);
 
   if (module)
     {
@@ -116,7 +118,12 @@ cogl_clear (const CoglColor *color, gulong buffers)
 
   COGL_NOTE (DRAW, "Clear begin");
 
-  cogl_clip_ensure ();
+  _cogl_journal_flush ();
+
+  /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
+   * as the material state) when flushing the clip stack, so should
+   * always be done first when preparing to draw. */
+  _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
 
   if (buffers & COGL_BUFFER_BIT_COLOR)
     {
@@ -264,6 +271,9 @@ cogl_set_backface_culling_enabled (gboolean setting)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  if (ctx->enable_backface_culling == setting)
+    return;
+
   /* Currently the journal can't track changes to backface culling state... */
   _cogl_journal_flush ();
 
@@ -278,6 +288,38 @@ cogl_get_backface_culling_enabled (void)
   return ctx->enable_backface_culling;
 }
 
+void
+_cogl_flush_face_winding (void)
+{
+  CoglFrontWinding winding;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  /* The front face winding doesn't matter if we aren't performing any
+   * backface culling... */
+  if (!ctx->enable_backface_culling)
+    return;
+
+  /* NB: We use a clockwise face winding order when drawing offscreen because
+   * all offscreen rendering is done upside down resulting in reversed winding
+   * for all triangles.
+   */
+  if (cogl_is_offscreen (_cogl_get_framebuffer ()))
+    winding = COGL_FRONT_WINDING_CLOCKWISE;
+  else
+    winding = COGL_FRONT_WINDING_COUNTER_CLOCKWISE;
+
+  if (winding != ctx->flushed_front_winding)
+    {
+
+      if (winding == COGL_FRONT_WINDING_CLOCKWISE)
+        GE (glFrontFace (GL_CW));
+      else
+        GE (glFrontFace (GL_CCW));
+      ctx->flushed_front_winding = winding;
+    }
+}
+
 void
 cogl_set_source_color (const CoglColor *color)
 {
@@ -286,248 +328,41 @@ cogl_set_source_color (const CoglColor *color)
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* In case cogl_set_source_texture was previously used... */
-  cogl_material_remove_layer (ctx->default_material, 0);
+  cogl_material_remove_layer (ctx->simple_material, 0);
 
   premultiplied = *color;
   cogl_color_premultiply (&premultiplied);
-  cogl_material_set_color (ctx->default_material, &premultiplied);
+  cogl_material_set_color (ctx->simple_material, &premultiplied);
 
-  cogl_set_source (ctx->default_material);
-}
-
-static void
-project_vertex (const CoglMatrix *modelview_matrix,
-		const CoglMatrix *projection_matrix,
-		float *vertex)
-{
-  int i;
-
-  /* Apply the modelview matrix */
-  cogl_matrix_transform_point (modelview_matrix,
-                               &vertex[0], &vertex[1],
-                               &vertex[2], &vertex[3]);
-  /* Apply the projection matrix */
-  cogl_matrix_transform_point (projection_matrix,
-                               &vertex[0], &vertex[1],
-                               &vertex[2], &vertex[3]);
-  /* Convert from homogenized coordinates */
-  for (i = 0; i < 4; i++)
-    vertex[i] /= vertex[3];
-}
-
-static void
-set_clip_plane (GLint plane_num,
-		const float *vertex_a,
-		const float *vertex_b)
-{
-#if defined (HAVE_COGL_GLES2) || defined (HAVE_COGL_GLES)
-  GLfloat plane[4];
-#else
-  GLdouble plane[4];
-#endif
-  GLfloat angle;
-  CoglMatrix inverse_projection;
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  /* Calculate the angle between the axes and the line crossing the
-     two points */
-  angle = atan2f (vertex_b[1] - vertex_a[1],
-                  vertex_b[0] - vertex_a[0]) * (180.0/G_PI);
-
-  _cogl_matrix_stack_push (ctx->modelview_stack);
-  /* Load the identity matrix and multiply by the reverse of the
-     projection matrix so we can specify the plane in screen
-     coordinates */
-  _cogl_matrix_stack_load_identity (ctx->modelview_stack);
-  cogl_matrix_init_from_array (&inverse_projection,
-                               ctx->inverse_projection);
-  _cogl_matrix_stack_multiply (ctx->modelview_stack, &inverse_projection);
-  /* Rotate about point a */
-  _cogl_matrix_stack_translate (ctx->modelview_stack,
-                                vertex_a[0], vertex_a[1], vertex_a[2]);
-  /* Rotate the plane by the calculated angle so that it will connect
-     the two points */
-  _cogl_matrix_stack_rotate (ctx->modelview_stack, angle, 0.0f, 0.0f, 1.0f);
-  _cogl_matrix_stack_translate (ctx->modelview_stack,
-                                -vertex_a[0], -vertex_a[1], -vertex_a[2]);
-
-  _cogl_flush_matrix_stacks ();
-
-  plane[0] = 0;
-  plane[1] = -1.0;
-  plane[2] = 0;
-  plane[3] = vertex_a[1];
-#if defined (HAVE_COGL_GLES2) || defined (HAVE_COGL_GLES)
-  GE( glClipPlanef (plane_num, plane) );
-#else
-  GE( glClipPlane (plane_num, plane) );
-#endif
-
-  _cogl_matrix_stack_pop (ctx->modelview_stack);
+  cogl_set_source (ctx->simple_material);
 }
 
 void
-_cogl_set_clip_planes (float x_offset,
-		       float y_offset,
-		       float width,
-		       float height)
+cogl_set_viewport (int x,
+                   int y,
+                   int width,
+                   int height)
 {
-  CoglMatrix modelview_matrix;
-  CoglMatrix projection_matrix;
+  CoglHandle framebuffer;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  float vertex_tl[4] = { x_offset, y_offset, 0, 1.0 };
-  float vertex_tr[4] = { x_offset + width, y_offset, 0, 1.0 };
-  float vertex_bl[4] = { x_offset, y_offset + height, 0, 1.0 };
-  float vertex_br[4] = { x_offset + width, y_offset + height,
-                        0, 1.0 };
+  framebuffer = _cogl_get_framebuffer ();
 
-  _cogl_matrix_stack_get (ctx->projection_stack, &projection_matrix);
-  _cogl_matrix_stack_get (ctx->modelview_stack, &modelview_matrix);
-
-  project_vertex (&modelview_matrix, &projection_matrix, vertex_tl);
-  project_vertex (&modelview_matrix, &projection_matrix, vertex_tr);
-  project_vertex (&modelview_matrix, &projection_matrix, vertex_bl);
-  project_vertex (&modelview_matrix, &projection_matrix, vertex_br);
-
-  /* If the order of the top and bottom lines is different from the
-     order of the left and right lines then the clip rect must have
-     been transformed so that the back is visible. We therefore need
-     to swap one pair of vertices otherwise all of the planes will be
-     the wrong way around */
-  if ((vertex_tl[0] < vertex_tr[0] ? 1 : 0)
-      != (vertex_bl[1] < vertex_tl[1] ? 1 : 0))
-    {
-      float temp[4];
-      memcpy (temp, vertex_tl, sizeof (temp));
-      memcpy (vertex_tl, vertex_tr, sizeof (temp));
-      memcpy (vertex_tr, temp, sizeof (temp));
-      memcpy (temp, vertex_bl, sizeof (temp));
-      memcpy (vertex_bl, vertex_br, sizeof (temp));
-      memcpy (vertex_br, temp, sizeof (temp));
-    }
-
-  set_clip_plane (GL_CLIP_PLANE0, vertex_tl, vertex_tr);
-  set_clip_plane (GL_CLIP_PLANE1, vertex_tr, vertex_br);
-  set_clip_plane (GL_CLIP_PLANE2, vertex_br, vertex_bl);
-  set_clip_plane (GL_CLIP_PLANE3, vertex_bl, vertex_tl);
+  _cogl_framebuffer_set_viewport (framebuffer,
+                                  x,
+                                  y,
+                                  width,
+                                  height);
 }
 
-void
-_cogl_add_stencil_clip (float x_offset,
-			float y_offset,
-			float width,
-			float height,
-			gboolean first)
-{
-  CoglHandle current_source;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  /* We don't log changes to the stencil buffer so need to flush any
-   * batched geometry before we start... */
-  _cogl_journal_flush ();
-
-  /* temporarily swap in our special stenciling material */
-  current_source = cogl_handle_ref (ctx->source_material);
-  cogl_set_source (ctx->stencil_material);
-
-  if (first)
-    {
-      GE( glEnable (GL_STENCIL_TEST) );
-
-      /* Initially disallow everything */
-      GE( glClearStencil (0) );
-      GE( glClear (GL_STENCIL_BUFFER_BIT) );
-
-      /* Punch out a hole to allow the rectangle */
-      GE( glStencilFunc (GL_NEVER, 0x1, 0x1) );
-      GE( glStencilOp (GL_REPLACE, GL_REPLACE, GL_REPLACE) );
-
-      cogl_rectangle (x_offset, y_offset,
-                      x_offset + width, y_offset + height);
-    }
-  else
-    {
-      /* Add one to every pixel of the stencil buffer in the
-	 rectangle */
-      GE( glStencilFunc (GL_NEVER, 0x1, 0x3) );
-      GE( glStencilOp (GL_INCR, GL_INCR, GL_INCR) );
-      cogl_rectangle (x_offset, y_offset,
-                      x_offset + width, y_offset + height);
-
-      /* make sure our rectangle hits the stencil buffer before we
-       * change the stencil operation */
-      _cogl_journal_flush ();
-
-      /* Subtract one from all pixels in the stencil buffer so that
-	 only pixels where both the original stencil buffer and the
-	 rectangle are set will be valid */
-      GE( glStencilOp (GL_DECR, GL_DECR, GL_DECR) );
-
-      _cogl_matrix_stack_push (ctx->projection_stack);
-      _cogl_matrix_stack_load_identity (ctx->projection_stack);
-
-      _cogl_matrix_stack_push (ctx->modelview_stack);
-      _cogl_matrix_stack_load_identity (ctx->modelview_stack);
-
-      cogl_rectangle (-1.0, -1.0, 1.0, 1.0);
-
-      _cogl_matrix_stack_pop (ctx->modelview_stack);
-      _cogl_matrix_stack_pop (ctx->projection_stack);
-    }
-
-  /* make sure our rectangles hit the stencil buffer before we restore
-   * the stencil function / operation */
-  _cogl_journal_flush ();
-
-  /* Restore the stencil mode */
-  GE( glStencilFunc (GL_EQUAL, 0x1, 0x1) );
-  GE( glStencilOp (GL_KEEP, GL_KEEP, GL_KEEP) );
-
-  /* restore the original source material */
-  cogl_set_source (current_source);
-  cogl_handle_unref (current_source);
-}
-
-void
-_cogl_disable_stencil_buffer (void)
-{
-  GE( glDisable (GL_STENCIL_TEST) );
-}
-
-void
-_cogl_enable_clip_planes (void)
-{
-  GE( glEnable (GL_CLIP_PLANE0) );
-  GE( glEnable (GL_CLIP_PLANE1) );
-  GE( glEnable (GL_CLIP_PLANE2) );
-  GE( glEnable (GL_CLIP_PLANE3) );
-}
-
-void
-_cogl_disable_clip_planes (void)
-{
-  GE( glDisable (GL_CLIP_PLANE3) );
-  GE( glDisable (GL_CLIP_PLANE2) );
-  GE( glDisable (GL_CLIP_PLANE1) );
-  GE( glDisable (GL_CLIP_PLANE0) );
-}
-
-/* XXX: This should be deprecated and Cogl should be left to manage
- * the glViewport automatically when switching draw buffers. */
+/* XXX: This should be deprecated, and we should expose a way to also
+ * specify an x and y viewport offset */
 void
 cogl_viewport (guint width,
 	       guint height)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  COGL_NOTE (MISC, "glViewport(0, 0, %u, %u)", width, height);
-  GE( glViewport (0, 0, width, height) );
-
-  ctx->viewport_width = width;
-  ctx->viewport_height = height;
+  cogl_set_viewport (0, 0, width, height);
 }
 
 void
@@ -540,10 +375,11 @@ _cogl_setup_viewport (guint width,
 {
   float z_camera;
   CoglMatrix projection_matrix;
+  CoglMatrixStack *modelview_stack;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_viewport (width, height);
+  cogl_set_viewport (0, 0, width, height);
 
   /* For Ortho projection.
    * _cogl_matrix_stack_ortho (projection_stack, 0, width, 0,  height, -1, 1);
@@ -593,11 +429,13 @@ _cogl_setup_viewport (guint width,
   cogl_get_projection_matrix (&projection_matrix);
   z_camera = 0.5 * projection_matrix.xx;
 
-  _cogl_matrix_stack_load_identity (ctx->modelview_stack);
-  _cogl_matrix_stack_translate (ctx->modelview_stack, -0.5f, -0.5f, -z_camera);
-  _cogl_matrix_stack_scale (ctx->modelview_stack,
+  modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_load_identity (modelview_stack);
+  _cogl_matrix_stack_translate (modelview_stack, -0.5f, -0.5f, -z_camera);
+  _cogl_matrix_stack_scale (modelview_stack,
                             1.0f / width, -1.0f / height, 1.0f / width);
-  _cogl_matrix_stack_translate (ctx->modelview_stack,
+  _cogl_matrix_stack_translate (modelview_stack,
                                 0.0f, -1.0 * height, 0.0f);
 }
 
@@ -606,9 +444,6 @@ cogl_get_features (void)
 {
   _COGL_GET_CONTEXT (ctx, 0);
 
-  if (!ctx->features_cached)
-    _cogl_features_init ();
-
   if (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS)
     ctx->feature_flags &= ~COGL_FEATURE_VBOS;
 
@@ -626,18 +461,24 @@ cogl_features_available (CoglFeatureFlags features)
   return (ctx->feature_flags & features) == features;
 }
 
-/* XXX: This function should be deprecated, and replaced with a
- * cogl_draw_buffer_get_size() API instead. We don't support offset
- * viewports, and you can't have floating point viewport sizes. */
+/* XXX: This function should either be replaced with one returning
+ * integers, or removed/deprecated and make the
+ * _cogl_framebuffer_get_viewport* functions public.
+ */
 void
 cogl_get_viewport (float v[4])
 {
+  CoglHandle framebuffer;
+  int viewport[4];
+  int i;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  v[0] = 0;
-  v[1] = 0;
-  v[2] = ctx->viewport_width;
-  v[3] = ctx->viewport_height;
+  framebuffer = _cogl_get_framebuffer ();
+  _cogl_framebuffer_get_viewport4fv (framebuffer, viewport);
+
+  for (i = 0; i < 4; i++)
+    v[i] = viewport[i];
 }
 
 void
@@ -729,14 +570,6 @@ cogl_disable_fog (void)
   glDisable (GL_FOG);
 }
 
-#if 0
-void
-cogl_flush_gl_state (int flags)
-{
-  _cogl_flush_matrix_stacks ();
-}
-#endif
-
 void
 cogl_flush (void)
 {
@@ -752,37 +585,52 @@ cogl_read_pixels (int x,
                   CoglPixelFormat format,
                   guint8 *pixels)
 {
-  GLint   viewport[4];
-  GLint   viewport_height;
-  int     rowstride = width * 4;
-  guint8  *temprow;
+  CoglHandle framebuffer;
+  int        framebuffer_height;
+  int        rowstride = width * 4;
+  guint8    *temprow;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   g_return_if_fail (format == COGL_PIXEL_FORMAT_RGBA_8888);
   g_return_if_fail (source == COGL_READ_PIXELS_COLOR_BUFFER);
 
-  temprow = g_alloca (rowstride * sizeof (guint8));
-
-  glGetIntegerv (GL_VIEWPORT, viewport);
-  viewport_height = viewport[3];
-
-  /* The y co-ordinate should be given in OpenGL's coordinate system
-     so 0 is the bottom row */
-  y = viewport_height - y - height;
-
-  /* Setup the pixel store parameters that may have been changed by
-     Cogl */
-  glPixelStorei (GL_PACK_ALIGNMENT, 4);
-#ifdef HAVE_COGL_GL
-  glPixelStorei (GL_PACK_ROW_LENGTH, 0);
-  glPixelStorei (GL_PACK_SKIP_PIXELS, 0);
-  glPixelStorei (GL_PACK_SKIP_ROWS, 0);
-#endif /* HAVE_COGL_GL */
-
   /* make sure any batched primitives get emitted to the GL driver before
    * issuing our read pixels... */
   cogl_flush ();
 
-  glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+  temprow = g_alloca (rowstride * sizeof (guint8));
+
+  framebuffer = _cogl_get_framebuffer ();
+
+  _cogl_framebuffer_flush_state (framebuffer, 0);
+
+  framebuffer_height = _cogl_framebuffer_get_height (framebuffer);
+
+  /* The y co-ordinate should be given in OpenGL's coordinate system
+   * so 0 is the bottom row
+   *
+   * NB: all offscreen rendering is done upside down so no conversion
+   * is necissary in this case.
+   */
+  if (!cogl_is_offscreen (framebuffer))
+    y = framebuffer_height - y - height;
+
+  /* Setup the pixel store parameters that may have been changed by
+     Cogl */
+  GE (glPixelStorei (GL_PACK_ALIGNMENT, 1));
+#ifdef HAVE_COGL_GL
+  GE (glPixelStorei (GL_PACK_ROW_LENGTH, 0));
+  GE (glPixelStorei (GL_PACK_SKIP_PIXELS, 0));
+  GE (glPixelStorei (GL_PACK_SKIP_ROWS, 0));
+#endif /* HAVE_COGL_GL */
+
+  GE (glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels));
+
+  /* NB: All offscreen rendering is done upside down so there is no need
+   * to flip in this case... */
+  if (cogl_is_offscreen (framebuffer))
+    return;
 
   /* TODO: consider using the GL_MESA_pack_invert extension in the future
    * to avoid this flip... */
@@ -807,7 +655,7 @@ void
 cogl_begin_gl (void)
 {
   CoglMaterialFlushOptions options;
-  gulong enable_flags;
+  gulong enable_flags = 0;
   int i;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -825,12 +673,13 @@ cogl_begin_gl (void)
   /* Flush all batched primitives */
   cogl_flush ();
 
-  /* Flush our clipping state to GL */
-  cogl_clip_ensure ();
-
-  /* Flush any client side matrix state */
-  _cogl_flush_matrix_stacks ();
-
+  /* Flush framebuffer state, including clip state, modelview and
+   * projection matrix state
+   *
+   * NB: _cogl_framebuffer_flush_state may disrupt various state (such
+   * as the material state) when flushing the clip stack, so should
+   * always be done first when preparing to draw. */
+  _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
 
   /* Setup the state for the current material */
 
@@ -852,6 +701,7 @@ cogl_begin_gl (void)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
   cogl_enable (enable_flags);
+  _cogl_flush_face_winding ();
 
   /* Disable all client texture coordinate arrays */
   for (i = 0; i < ctx->n_texcoord_arrays_enabled; i++)
@@ -940,36 +790,41 @@ _cogl_destroy_texture_units (void)
 void
 cogl_push_matrix (void)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_push (ctx->modelview_stack);
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_push (modelview_stack);
 }
 
 void
 cogl_pop_matrix (void)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_pop (ctx->modelview_stack);
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_pop (modelview_stack);
 }
 
 void
 cogl_scale (float x, float y, float z)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_scale (ctx->modelview_stack, x, y, z);
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_scale (modelview_stack, x, y, z);
 }
 
 void
 cogl_translate (float x, float y, float z)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_translate (ctx->modelview_stack, x, y, z);
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_translate (modelview_stack, x, y, z);
 }
 
 void
 cogl_rotate (float angle, float x, float y, float z)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_rotate (ctx->modelview_stack, angle, x, y, z);
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_rotate (modelview_stack, angle, x, y, z);
 }
 
 void
@@ -996,35 +851,20 @@ cogl_frustum (float        left,
 	      float        z_near,
 	      float        z_far)
 {
-  float c, d;
+  CoglMatrixStack *projection_stack =
+    _cogl_framebuffer_get_projection_stack (_cogl_get_framebuffer ());
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  _cogl_matrix_stack_load_identity (ctx->projection_stack);
+  _cogl_matrix_stack_load_identity (projection_stack);
 
-  _cogl_matrix_stack_frustum (ctx->projection_stack,
+  _cogl_matrix_stack_frustum (projection_stack,
                               left,
                               right,
                               bottom,
                               top,
                               z_near,
                               z_far);
-
-  /* Calculate and store the inverse of the matrix */
-  memset (ctx->inverse_projection, 0, sizeof (float) * 16);
-
-  c = - (z_far + z_near) /  (z_far - z_near);
-  d = - (2 * (z_far * z_near)) /  (z_far - z_near);
-
-#define M(row,col)  ctx->inverse_projection[col*4+row]
-  M(0,0) =  (right - left) /  (2 * z_near);
-  M(0,3) =  (right + left) /  (2 * z_near);
-  M(1,1) =  (top - bottom) /  (2 * z_near);
-  M(1,3) =  (top + bottom) /  (2 * z_near);
-  M(2,3) = -1.0;
-  M(3,2) = 1.0 / d;
-  M(3,3) = c / d;
-#undef M
 }
 
 void
@@ -1036,65 +876,66 @@ cogl_ortho (float left,
 	    float z_far)
 {
   CoglMatrix ortho;
+  CoglMatrixStack *projection_stack =
+    _cogl_framebuffer_get_projection_stack (_cogl_get_framebuffer ());
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   cogl_matrix_init_identity (&ortho);
   cogl_matrix_ortho (&ortho, left, right, bottom, top, z_near, z_far);
-  _cogl_matrix_stack_set (ctx->projection_stack, &ortho);
-
-  /* Calculate and store the inverse of the matrix */
-  memset (ctx->inverse_projection, 0, sizeof (float) * 16);
-
-#define M(row,col)  ctx->inverse_projection[col*4+row]
-  M(0,0) =  1.0 / ortho.xx;
-  M(0,3) =  -ortho.xw;
-  M(1,1) =  1.0 / ortho.yy;
-  M(1,3) =  -ortho.yw;
-  M(2,2) =  1.0 / ortho.zz;
-  M(2,3) =  -ortho.zw;
-  M(3,3) =  1.0;
-#undef M
+  _cogl_matrix_stack_set (projection_stack, &ortho);
 }
 
 void
 cogl_get_modelview_matrix (CoglMatrix *matrix)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_get (ctx->modelview_stack, matrix);
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_get (modelview_stack, matrix);
+  _COGL_MATRIX_DEBUG_PRINT (matrix);
 }
 
 void
 cogl_set_modelview_matrix (CoglMatrix *matrix)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_set (ctx->modelview_stack, matrix);
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_set (modelview_stack, matrix);
+  _COGL_MATRIX_DEBUG_PRINT (matrix);
 }
 
 void
 cogl_get_projection_matrix (CoglMatrix *matrix)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_get (ctx->projection_stack, matrix);
+  CoglMatrixStack *projection_stack =
+    _cogl_framebuffer_get_projection_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_get (projection_stack, matrix);
+  _COGL_MATRIX_DEBUG_PRINT (matrix);
 }
 
 void
 cogl_set_projection_matrix (CoglMatrix *matrix)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_set (ctx->projection_stack, matrix);
+  CoglMatrixStack *projection_stack =
+    _cogl_framebuffer_get_projection_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_set (projection_stack, matrix);
 
   /* FIXME: Update the inverse projection matrix!! Presumably use
    * of clip planes must currently be broken if this API is used. */
+  _COGL_MATRIX_DEBUG_PRINT (matrix);
 }
 
-void
-_cogl_flush_matrix_stacks (void)
+CoglClipStackState *
+_cogl_get_clip_state (void)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  _cogl_matrix_stack_flush_to_gl (ctx->projection_stack,
-                                  COGL_MATRIX_PROJECTION);
-  _cogl_matrix_stack_flush_to_gl (ctx->modelview_stack,
-                                  COGL_MATRIX_MODELVIEW);
+  CoglHandle framebuffer;
+
+  framebuffer = _cogl_get_framebuffer ();
+  return _cogl_framebuffer_get_clip_state (framebuffer);
 }
 
+GQuark
+_cogl_driver_error_quark (void)
+{
+  return g_quark_from_static_string ("cogl-driver-error-quark");
+}
